# coding: utf-8

"""
    Coinbase Developer Platform APIs

    The Coinbase Developer Platform APIs - leading the world's transition onchain.

    The version of the OpenAPI document: 2.0.0
    Contact: cdp@coinbase.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from cdp.openapi_client.models.list_evm_token_balances_network import ListEvmTokenBalancesNetwork

class TestListEvmTokenBalancesNetwork(unittest.TestCase):
    """ListEvmTokenBalancesNetwork unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def testListEvmTokenBalancesNetwork(self):
        """Test ListEvmTokenBalancesNetwork"""
        # inst = ListEvmTokenBalancesNetwork()

if __name__ == '__main__':
    unittest.main()
