# coding: utf-8

"""
    Coinbase Developer Platform APIs

    The Coinbase Developer Platform APIs - leading the world's transition onchain.

    The version of the OpenAPI document: 2.0.0
    Contact: cdp@coinbase.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from cdp.openapi_client.api.policy_engine_api import PolicyEngineApi


class TestPolicyEngineApi(unittest.IsolatedAsyncioTestCase):
    """PolicyEngineApi unit test stubs"""

    async def asyncSetUp(self) -> None:
        self.api = PolicyEngineApi()

    async def asyncTearDown(self) -> None:
        await self.api.api_client.close()

    async def test_create_policy(self) -> None:
        """Test case for create_policy

        Create a policy
        """
        pass

    async def test_delete_policy(self) -> None:
        """Test case for delete_policy

        Delete a policy
        """
        pass

    async def test_get_policy_by_id(self) -> None:
        """Test case for get_policy_by_id

        Get a policy by ID
        """
        pass

    async def test_list_policies(self) -> None:
        """Test case for list_policies

        List policies
        """
        pass

    async def test_update_policy(self) -> None:
        """Test case for update_policy

        Update a policy
        """
        pass


if __name__ == '__main__':
    unittest.main()
