# coding: utf-8

"""
    Coinbase Developer Platform APIs

    The Coinbase Developer Platform APIs - leading the world's transition onchain.

    The version of the OpenAPI document: 2.0.0
    Contact: cdp@coinbase.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from cdp.openapi_client.api.solana_accounts_api import SolanaAccountsApi


class TestSolanaAccountsApi(unittest.IsolatedAsyncioTestCase):
    """SolanaAccountsApi unit test stubs"""

    async def asyncSetUp(self) -> None:
        self.api = SolanaAccountsApi()

    async def asyncTearDown(self) -> None:
        await self.api.api_client.close()

    async def test_create_solana_account(self) -> None:
        """Test case for create_solana_account

        Create a Solana account
        """
        pass

    async def test_get_solana_account(self) -> None:
        """Test case for get_solana_account

        Get a Solana account by address
        """
        pass

    async def test_get_solana_account_by_name(self) -> None:
        """Test case for get_solana_account_by_name

        Get a Solana account by name
        """
        pass

    async def test_list_solana_accounts(self) -> None:
        """Test case for list_solana_accounts

        List Solana accounts or get account by name
        """
        pass

    async def test_sign_solana_message(self) -> None:
        """Test case for sign_solana_message

        Sign a message
        """
        pass

    async def test_sign_solana_transaction(self) -> None:
        """Test case for sign_solana_transaction

        Sign a transaction
        """
        pass

    async def test_update_solana_account(self) -> None:
        """Test case for update_solana_account

        Update a Solana account
        """
        pass


if __name__ == '__main__':
    unittest.main()
