# coding: utf-8

"""
    Coinbase Developer Platform APIs

    The Coinbase Developer Platform APIs - leading the world's transition onchain.

    The version of the OpenAPI document: 2.0.0
    Contact: cdp@coinbase.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from cdp.openapi_client.models.evm_user_operation import EvmUserOperation

class TestEvmUserOperation(unittest.TestCase):
    """EvmUserOperation unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional) -> EvmUserOperation:
        """Test EvmUserOperation
            include_optional is a boolean, when False only required
            params are included, when True both required and
            optional params are included """
        # uncomment below to create an instance of `EvmUserOperation`
        """
        model = EvmUserOperation()
        if include_optional:
            return EvmUserOperation(
                network = 'base-sepolia',
                user_op_hash = '0x1234567890abcdef1234567890abcdef1234567890abcdef1234567890abcdef',
                calls = [{"to":"0xa0b86991c6218b36c1d19d4a2e9eb0ce3606eb48","value":"0","data":"0xa9059cbb000000000000000000000000fc807d1be4997e5c7b33e4d8d57e60c5b0f02b1a0000000000000000000000000000000000000000000000000000000000000064"},{"to":"0xdac17f958d2ee523a2206206994597c13d831ec7","value":"1000000000000000","data":"0x"}],
                status = 'pending',
                transaction_hash = '0x0123456789abcdef0123456789abcdef0123456789abcdef0123456789abcdef'
            )
        else:
            return EvmUserOperation(
                network = 'base-sepolia',
                user_op_hash = '0x1234567890abcdef1234567890abcdef1234567890abcdef1234567890abcdef',
                calls = [{"to":"0xa0b86991c6218b36c1d19d4a2e9eb0ce3606eb48","value":"0","data":"0xa9059cbb000000000000000000000000fc807d1be4997e5c7b33e4d8d57e60c5b0f02b1a0000000000000000000000000000000000000000000000000000000000000064"},{"to":"0xdac17f958d2ee523a2206206994597c13d831ec7","value":"1000000000000000","data":"0x"}],
                status = 'pending',
        )
        """

    def testEvmUserOperation(self):
        """Test EvmUserOperation"""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)

if __name__ == '__main__':
    unittest.main()
