# coding: utf-8

"""
    Coinbase Developer Platform APIs

    The Coinbase Developer Platform APIs - leading the world's transition onchain.

    The version of the OpenAPI document: 2.0.0
    Contact: cdp@coinbase.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from cdp.openapi_client.models.sign_evm_message_criteria_inner import SignEvmMessageCriteriaInner

class TestSignEvmMessageCriteriaInner(unittest.TestCase):
    """SignEvmMessageCriteriaInner unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional) -> SignEvmMessageCriteriaInner:
        """Test SignEvmMessageCriteriaInner
            include_optional is a boolean, when False only required
            params are included, when True both required and
            optional params are included """
        # uncomment below to create an instance of `SignEvmMessageCriteriaInner`
        """
        model = SignEvmMessageCriteriaInner()
        if include_optional:
            return SignEvmMessageCriteriaInner(
                type = 'evmMessage',
                match = '^hello ([a-z]+)$'
            )
        else:
            return SignEvmMessageCriteriaInner(
                type = 'evmMessage',
                match = '^hello ([a-z]+)$',
        )
        """

    def testSignEvmMessageCriteriaInner(self):
        """Test SignEvmMessageCriteriaInner"""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)

if __name__ == '__main__':
    unittest.main()
