# coding: utf-8

"""
    Coinbase Developer Platform APIs

    The Coinbase Developer Platform APIs - leading the world's transition onchain.

    The version of the OpenAPI document: 2.0.0
    Contact: cdp@coinbase.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from cdp.openapi_client.api.payments_alpha_api import PaymentsAlphaApi


class TestPaymentsAlphaApi(unittest.IsolatedAsyncioTestCase):
    """PaymentsAlphaApi unit test stubs"""

    async def asyncSetUp(self) -> None:
        self.api = PaymentsAlphaApi()

    async def asyncTearDown(self) -> None:
        await self.api.api_client.close()

    async def test_create_payment_transfer_quote(self) -> None:
        """Test case for create_payment_transfer_quote

        Create a transfer quote
        """
        pass

    async def test_execute_payment_transfer_quote(self) -> None:
        """Test case for execute_payment_transfer_quote

        Execute a transfer quote
        """
        pass

    async def test_get_crypto_rails(self) -> None:
        """Test case for get_crypto_rails

        Get the crypto rails
        """
        pass

    async def test_get_payment_methods(self) -> None:
        """Test case for get_payment_methods

        Get the fiat payment methods
        """
        pass

    async def test_get_payment_transfer(self) -> None:
        """Test case for get_payment_transfer

        Get a transfer by ID
        """
        pass


if __name__ == '__main__':
    unittest.main()
