# coding: utf-8

"""
    Coinbase Developer Platform APIs

    The Coinbase Developer Platform APIs - leading the world's transition onchain.

    The version of the OpenAPI document: 2.0.0
    Contact: cdp@coinbase.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from cdp.openapi_client.models.list_evm_token_balances200_response import ListEvmTokenBalances200Response

class TestListEvmTokenBalances200Response(unittest.TestCase):
    """ListEvmTokenBalances200Response unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional) -> ListEvmTokenBalances200Response:
        """Test ListEvmTokenBalances200Response
            include_optional is a boolean, when False only required
            params are included, when True both required and
            optional params are included """
        # uncomment below to create an instance of `ListEvmTokenBalances200Response`
        """
        model = ListEvmTokenBalances200Response()
        if include_optional:
            return ListEvmTokenBalances200Response(
                next_page_token = 'eyJsYXN0X2lkIjogImFiYzEyMyIsICJ0aW1lc3RhbXAiOiAxNzA3ODIzNzAxfQ==',
                balances = [{"amount":{"amount":"1250000000000000000","decimals":18},"token":{"network":"base","symbol":"ETH","name":"ether","contractAddress":"0xEeeeeEeeeEeEeeEeEeEeeEEEeeeeEeeeeeeeEEeE"}},{"amount":{"amount":"123456","decimals":6},"token":{"network":"base","symbol":"USDC","name":"USD Coin","contractAddress":"0x833589fCD6eDb6E08f4c7C32D4f71b54bdA02913"}}]
            )
        else:
            return ListEvmTokenBalances200Response(
                balances = [{"amount":{"amount":"1250000000000000000","decimals":18},"token":{"network":"base","symbol":"ETH","name":"ether","contractAddress":"0xEeeeeEeeeEeEeeEeEeEeeEEEeeeeEeeeeeeeEEeE"}},{"amount":{"amount":"123456","decimals":6},"token":{"network":"base","symbol":"USDC","name":"USD Coin","contractAddress":"0x833589fCD6eDb6E08f4c7C32D4f71b54bdA02913"}}],
        )
        """

    def testListEvmTokenBalances200Response(self):
        """Test ListEvmTokenBalances200Response"""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)

if __name__ == '__main__':
    unittest.main()
