# coding: utf-8

"""
    Coinbase Developer Platform APIs

    The Coinbase Developer Platform APIs - leading the world's transition onchain.

    The version of the OpenAPI document: 2.0.0
    Contact: cdp@coinbase.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from cdp.openapi_client.models.list_policies200_response import ListPolicies200Response

class TestListPolicies200Response(unittest.TestCase):
    """ListPolicies200Response unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional) -> ListPolicies200Response:
        """Test ListPolicies200Response
            include_optional is a boolean, when False only required
            params are included, when True both required and
            optional params are included """
        # uncomment below to create an instance of `ListPolicies200Response`
        """
        model = ListPolicies200Response()
        if include_optional:
            return ListPolicies200Response(
                next_page_token = 'eyJsYXN0X2lkIjogImFiYzEyMyIsICJ0aW1lc3RhbXAiOiAxNzA3ODIzNzAxfQ==',
                policies = [
                    cdp.openapi_client.models.policy.Policy(
                        id = '123e4567-e89b-12d3-a456-426614174000', 
                        description = 'Default policy', 
                        scope = 'project', 
                        rules = [{"action":"accept","operation":"signEvmTransaction","criteria":[{"type":"ethValue","ethValue":"1000000000000000000","operator":"<="},{"type":"evmAddress","addresses":["0x742d35Cc6634C0532925a3b844Bc454e4438f44e","0x1234567890123456789012345678901234567890"],"operator":"in"}]},{"action":"accept","operation":"signSolTransaction","criteria":[{"type":"solAddress","addresses":["HpabPRRCFbBKSuJr5PdkVvQc85FyxyTWkFM2obBRSvHT"],"operator":"in"}]}], 
                        created_at = '2025-03-25T12:00:00Z', 
                        updated_at = '2025-03-26T12:00:00Z', )
                    ]
            )
        else:
            return ListPolicies200Response(
                policies = [
                    cdp.openapi_client.models.policy.Policy(
                        id = '123e4567-e89b-12d3-a456-426614174000', 
                        description = 'Default policy', 
                        scope = 'project', 
                        rules = [{"action":"accept","operation":"signEvmTransaction","criteria":[{"type":"ethValue","ethValue":"1000000000000000000","operator":"<="},{"type":"evmAddress","addresses":["0x742d35Cc6634C0532925a3b844Bc454e4438f44e","0x1234567890123456789012345678901234567890"],"operator":"in"}]},{"action":"accept","operation":"signSolTransaction","criteria":[{"type":"solAddress","addresses":["HpabPRRCFbBKSuJr5PdkVvQc85FyxyTWkFM2obBRSvHT"],"operator":"in"}]}], 
                        created_at = '2025-03-25T12:00:00Z', 
                        updated_at = '2025-03-26T12:00:00Z', )
                    ],
        )
        """

    def testListPolicies200Response(self):
        """Test ListPolicies200Response"""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)

if __name__ == '__main__':
    unittest.main()
