# coding: utf-8

"""
    Coinbase Developer Platform APIs

    The Coinbase Developer Platform APIs - leading the world's transition onchain.

    The version of the OpenAPI document: 2.0.0
    Contact: cdp@coinbase.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from cdp.openapi_client.api.evm_swaps_api import EVMSwapsApi


class TestEVMSwapsApi(unittest.IsolatedAsyncioTestCase):
    """EVMSwapsApi unit test stubs"""

    async def asyncSetUp(self) -> None:
        self.api = EVMSwapsApi()

    async def asyncTearDown(self) -> None:
        await self.api.api_client.close()

    async def test_create_evm_swap_quote(self) -> None:
        """Test case for create_evm_swap_quote

        Create a swap quote
        """
        pass

    async def test_get_evm_swap_price(self) -> None:
        """Test case for get_evm_swap_price

        Get a price estimate for a swap
        """
        pass


if __name__ == '__main__':
    unittest.main()
