# coding: utf-8

"""
    Coinbase Developer Platform APIs

    The Coinbase Developer Platform APIs - leading the world's transition onchain.

    The version of the OpenAPI document: 2.0.0
    Contact: cdp@coinbase.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field
from typing import Any, ClassVar, Dict, List, Optional
from cdp.openapi_client.models.token_fee import TokenFee
from typing import Optional, Set
from typing_extensions import Self

class CommonSwapResponseFees(BaseModel):
    """
    The estimated fees for the swap.
    """ # noqa: E501
    gas_fee: Optional[TokenFee] = Field(description="The estimated gas fee for the swap.", alias="gasFee")
    protocol_fee: Optional[TokenFee] = Field(description="The estimated protocol fee for the swap.", alias="protocolFee")
    __properties: ClassVar[List[str]] = ["gasFee", "protocolFee"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of CommonSwapResponseFees from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of gas_fee
        if self.gas_fee:
            _dict['gasFee'] = self.gas_fee.to_dict()
        # override the default output from pydantic by calling `to_dict()` of protocol_fee
        if self.protocol_fee:
            _dict['protocolFee'] = self.protocol_fee.to_dict()
        # set to None if gas_fee (nullable) is None
        # and model_fields_set contains the field
        if self.gas_fee is None and "gas_fee" in self.model_fields_set:
            _dict['gasFee'] = None

        # set to None if protocol_fee (nullable) is None
        # and model_fields_set contains the field
        if self.protocol_fee is None and "protocol_fee" in self.model_fields_set:
            _dict['protocolFee'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of CommonSwapResponseFees from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "gasFee": TokenFee.from_dict(obj["gasFee"]) if obj.get("gasFee") is not None else None,
            "protocolFee": TokenFee.from_dict(obj["protocolFee"]) if obj.get("protocolFee") is not None else None
        })
        return _obj


