# coding: utf-8

"""
    Coinbase Developer Platform APIs

    The Coinbase Developer Platform APIs - leading the world's transition onchain.

    The version of the OpenAPI document: 2.0.0
    Contact: cdp@coinbase.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from cdp.openapi_client.api.evm_accounts_api import EVMAccountsApi


class TestEVMAccountsApi(unittest.IsolatedAsyncioTestCase):
    """EVMAccountsApi unit test stubs"""

    async def asyncSetUp(self) -> None:
        self.api = EVMAccountsApi()

    async def asyncTearDown(self) -> None:
        await self.api.api_client.close()

    async def test_create_evm_account(self) -> None:
        """Test case for create_evm_account

        Create an EVM account
        """
        pass

    async def test_get_evm_account(self) -> None:
        """Test case for get_evm_account

        Get an EVM account by address
        """
        pass

    async def test_get_evm_account_by_name(self) -> None:
        """Test case for get_evm_account_by_name

        Get an EVM account by name
        """
        pass

    async def test_import_evm_account(self) -> None:
        """Test case for import_evm_account

        Import an EVM account
        """
        pass

    async def test_list_evm_accounts(self) -> None:
        """Test case for list_evm_accounts

        List EVM accounts
        """
        pass

    async def test_send_evm_transaction(self) -> None:
        """Test case for send_evm_transaction

        Send a transaction
        """
        pass

    async def test_sign_evm_hash(self) -> None:
        """Test case for sign_evm_hash

        Sign a hash
        """
        pass

    async def test_sign_evm_message(self) -> None:
        """Test case for sign_evm_message

        Sign an EIP-191 message
        """
        pass

    async def test_sign_evm_transaction(self) -> None:
        """Test case for sign_evm_transaction

        Sign a transaction
        """
        pass

    async def test_sign_evm_typed_data(self) -> None:
        """Test case for sign_evm_typed_data

        Sign EIP-712 typed data
        """
        pass

    async def test_update_evm_account(self) -> None:
        """Test case for update_evm_account

        Update an EVM account
        """
        pass


if __name__ == '__main__':
    unittest.main()
