# coding: utf-8

"""
    Coinbase Developer Platform APIs

    The Coinbase Developer Platform APIs - leading the world's transition onchain.

    The version of the OpenAPI document: 2.0.0
    Contact: cdp@coinbase.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from cdp.openapi_client.models.send_evm_transaction_request import SendEvmTransactionRequest

class TestSendEvmTransactionRequest(unittest.TestCase):
    """SendEvmTransactionRequest unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional) -> SendEvmTransactionRequest:
        """Test SendEvmTransactionRequest
            include_optional is a boolean, when False only required
            params are included, when True both required and
            optional params are included """
        # uncomment below to create an instance of `SendEvmTransactionRequest`
        """
        model = SendEvmTransactionRequest()
        if include_optional:
            return SendEvmTransactionRequest(
                network = 'base-sepolia',
                transaction = '0xf86b098505d21dba00830334509431415daf58e2c6b7323b4c58712fd92952145da79018080'
            )
        else:
            return SendEvmTransactionRequest(
                network = 'base-sepolia',
                transaction = '0xf86b098505d21dba00830334509431415daf58e2c6b7323b4c58712fd92952145da79018080',
        )
        """

    def testSendEvmTransactionRequest(self):
        """Test SendEvmTransactionRequest"""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)

if __name__ == '__main__':
    unittest.main()
