# coding: utf-8

"""
    Coinbase Developer Platform APIs

    The Coinbase Developer Platform APIs - leading the world's transition onchain.

    The version of the OpenAPI document: 2.0.0
    Contact: cdp@coinbase.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from cdp.openapi_client.models.create_evm_swap_quote_request import CreateEvmSwapQuoteRequest

class TestCreateEvmSwapQuoteRequest(unittest.TestCase):
    """CreateEvmSwapQuoteRequest unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional) -> CreateEvmSwapQuoteRequest:
        """Test CreateEvmSwapQuoteRequest
            include_optional is a boolean, when False only required
            params are included, when True both required and
            optional params are included """
        # uncomment below to create an instance of `CreateEvmSwapQuoteRequest`
        """
        model = CreateEvmSwapQuoteRequest()
        if include_optional:
            return CreateEvmSwapQuoteRequest(
                network = 'base',
                to_token = '0x7F5c764cBc14f9669B88837ca1490cCa17c31607',
                from_token = '0x6B175474E89094C44Da98b954EedeAC495271d0F',
                from_amount = '1000000000000000000',
                taker = '0xAc0974bec39a17e36ba4a6b4d238ff944bacb478',
                signer_address = '0x922f49447d8a07e3bd95bd0d56f35241523fbab8',
                gas_price = '1000000000',
                slippage_bps = 100
            )
        else:
            return CreateEvmSwapQuoteRequest(
                network = 'base',
                to_token = '0x7F5c764cBc14f9669B88837ca1490cCa17c31607',
                from_token = '0x6B175474E89094C44Da98b954EedeAC495271d0F',
                from_amount = '1000000000000000000',
                taker = '0xAc0974bec39a17e36ba4a6b4d238ff944bacb478',
        )
        """

    def testCreateEvmSwapQuoteRequest(self):
        """Test CreateEvmSwapQuoteRequest"""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)

if __name__ == '__main__':
    unittest.main()
