# coding: utf-8

"""
    Coinbase Developer Platform APIs

    The Coinbase Developer Platform APIs - leading the world's transition onchain.

    The version of the OpenAPI document: 2.0.0
    Contact: cdp@coinbase.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictStr
from typing import Any, ClassVar, Dict, List
from cdp.openapi_client.models.eip712_domain import EIP712Domain
from typing import Optional, Set
from typing_extensions import Self

class EIP712Message(BaseModel):
    """
    The message to sign using EIP-712.
    """ # noqa: E501
    domain: EIP712Domain
    types: Dict[str, Any] = Field(description="A mapping of struct names to an array of type objects (name + type). Each key corresponds to a type name (e.g., \"`EIP712Domain`\", \"`PermitTransferFrom`\"). ")
    primary_type: StrictStr = Field(description="The primary type of the message. This is the name of the struct in the `types` object that is the root of the message.", alias="primaryType")
    message: Dict[str, Any] = Field(description="The message to sign. The structure of this message must match the `primaryType` struct in the `types` object.")
    __properties: ClassVar[List[str]] = ["domain", "types", "primaryType", "message"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of EIP712Message from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of domain
        if self.domain:
            _dict['domain'] = self.domain.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of EIP712Message from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "domain": EIP712Domain.from_dict(obj["domain"]) if obj.get("domain") is not None else None,
            "types": obj.get("types"),
            "primaryType": obj.get("primaryType"),
            "message": obj.get("message")
        })
        return _obj


