# coding: utf-8

"""
    Coinbase Developer Platform APIs

    The Coinbase Developer Platform APIs - leading the world's transition onchain.

    The version of the OpenAPI document: 2.0.0
    Contact: cdp@coinbase.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from cdp.openapi_client.models.payment_method_request import PaymentMethodRequest

class TestPaymentMethodRequest(unittest.TestCase):
    """PaymentMethodRequest unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional) -> PaymentMethodRequest:
        """Test PaymentMethodRequest
            include_optional is a boolean, when False only required
            params are included, when True both required and
            optional params are included """
        # uncomment below to create an instance of `PaymentMethodRequest`
        """
        model = PaymentMethodRequest()
        if include_optional:
            return PaymentMethodRequest(
                id = '8e03978e-40d5-43e8-bc93-6894a57f9324'
            )
        else:
            return PaymentMethodRequest(
                id = '8e03978e-40d5-43e8-bc93-6894a57f9324',
        )
        """

    def testPaymentMethodRequest(self):
        """Test PaymentMethodRequest"""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)

if __name__ == '__main__':
    unittest.main()
