"""Specifies package level constants used throughout the package."""

# SDK_DEFAULT_SOURCE (str): Denotes the default source for the Python SDK.
SDK_DEFAULT_SOURCE = "sdk"

# ERROR_DOCS_PAGE_URL (str): The URL to the Coinbase CDP API error docs page.
ERROR_DOCS_PAGE_URL = "https://docs.cloud.coinbase.com/cdp/docs/errors"
